function [viterbiPitch,statePath,pObservations,pTransitions] = ...
    CalculatePitch(fileNameOrFileList, pitchModel, groundTruthPitch,...
        forceCalculation)
% CALCULATEPITCH Calculate the pitch of one file (or list of files)
%
% function [viterbiPitch,statePath,pObservations,pTransitions] = ...
%    CalculatePitch(fileNameOrFileList, pitchModel, groundTruthPitch)
% Calculate the pitch for one file. The file can be specified with either a
% filename, or a filestruct (as comes from FindPitchFiles).  The pitchModel
% describes the pitch algorithm.  The groundTruthPitch is an optional
% argument that is used for plotting the final summary.  The
% groundTruthPitch is in Hz, and is sampled at 100Hz.
% All time-varying inputs and outputs have time going horizontally. 


if nargin < 3
    groundTruthPitch = [];
end

global gForceCorrelogramCalculation
if nargin < 4
    if exist('gForceCorrelogramCalculation', 'var') && ~isempty(gForceCorrelogramCalculation)
        forceCalculation = gForceCorrelogramCalculation;
    else
        forceCalculation = 0;
    end
end

fileList = ComputeAllFeatures(fileNameOrFileList, pitchModel, 1, forceCalculation);

figure(1);
[viterbiPitch,statePath,pObservations,pTransitions] = ...
    CalculateViterbiPitch(fileList(1), pitchModel);

if exist('groundTruthPitch','var') && ~isempty(groundTruthPitch)
    figure(2);
    subplot(2,1,1);
    imagesc(pObservations.^0.5);
    xlabel('Time (centiseconds)'); ylabel('Pitch Class');
    title('Pitch Observation Probabilities');
    subplot(2,1,2);
    plot(1:length(viterbiPitch), viterbiPitch, 'x', ...
        1:length(groundTruthPitch), groundTruthPitch);
    xlabel('Time (centiseconds'); ylabel('Pitch (Hz)');
    title('Ground truth (-) and Viterbi Pitch Estimates (x) ');
    % legend('Viterbi Estimate', 'Ground Truth');
    a = axis();
    axis([1 size(pObservations,2) a(3) a(4)]);
end

if PlaySounds()
    if ischar(fileNameOrFileList)
        [soundData, soundFs] = audioread(fileNameOrFileList);
    else
        [soundData, soundFs] = audioread(fileNameOrFileList(1).waveFile);
    end
    soundsc(soundData, soundFs);
end


function res = PlaySounds()
global gPlayPitchSounds
if exist('gPlayPitchSounds','var') && ~isempty(gPlayPitchSounds) && ~gPlayPitchSounds
    res = 0;
else
    address = java.net.InetAddress.getLocalHost();
    res = ~strncmp(address, 'stg58', 5);
end
