function [principalComponents, eigenValues, meanValues] = ComputePCA(data)
% Compute the principal components to explain some data.  Each column is
% one sample of the dataset.  This routine returns the principal
% components,their corresponding eigenvalues, and the mean that is first
% subtracted from each data point before PCA analysis.
if nargin < 1
    D = 2;
    N = 10000;
    ScalingMatrix = [2 0; 0 1; 0 0];
    c = cos(pi/4); s = sin(pi/4);
    RotationMatrix = [c 0 s; -s 0 c; 0 1 0];
    data = RotationMatrix * ScalingMatrix * randn(D, N);
end

nDimensions = size(data,1);
nSamples = size(data,2);

meanValues = mean(data, 2);
data = data - repmat(meanValues, 1, nSamples);

if 2*nSamples < nDimensions
    %%
    % From: http://cs.ait.ac.th/~mdailey/matlab/pc_evectors.m
    % Based on Turk and Pentland's hack for nSamples << nDimensions
    % not checked...
    L = data' * data;
    [eVectors, D] = eig(L);
    principalComponents = data * eVectors;
    componentAmplitude = sqrt(sum(principalComponents.^2));
    principalComponents = principalComponents ./ ...
        repmat(componentAmplitude, size(principalComponents,1),1);
    eigenValues = diag(D) / (nSamples-1);
else
    
    %% www.cs.stevens.edu/~mordohai/classes/cs559_s09/PCA_in_MATLAB.pdf
    Y = data' / sqrt(nSamples-1);
    
    [u,s,principalComponents] = svd(Y, 'econ');
    eigenValues = diag(s);
end

[eigenValues, eigenIndex] = sort(eigenValues, 'descend');
principalComponents = principalComponents(:,eigenIndex);

if nargin < 1
    signals = principalComponents' * data;
    plot(signals(1,:), signals(2,:), 'x');
    axis square
end

