function [transitionProbabilities, classProbabilities, allTransitions] = ...
    FindTransitionProbabilities(fileList)
% FINDTRANSITIONPROBABILITIES Summarize pitch transition probabilities for a set of pitch data
%
% [transitionProbabilities, classProbabilities] = FindTransitionProbabilities(fileList)
% Find the pitch-class transition probabilities for a set of test data.
% This is needed for the Viterbi decoder. This routine uses the
% AccumulateTrantions() routine to do the basic data collection.
% 
% This routine returns 
% 1) a state-transtion matrix normalized so that the sum of 
%   each row (one old state) is equal to 1.
% 2) a class probability matrix, regularized so that the no class has zero
%   probability.
% 3) the raw transition matrix (fromState, toState);
%%
global gPitchClassRegularizerRatio       % Added by Malcolm 10/06/2012
if exist('gPitchClassRegularizerRatio','var') && ~isempty(gPitchClassRegularizerRatio)
    pitchClassRegularizerRatio = gPitchClassRegularizerRatio;
else
    pitchClassRegularizerRatio = 1e-1;
end


%%

maxStates = max(QuantizePitches([10:10:1000]));
allTransitions = 0;
for i=1:length(fileList)
    q = fileList(i).pitchClass;
    allTransitions = AccumulateTransitions(q, allTransitions);
    % allTransitions(1,1) = nan;
    if 1
       clf
       imagesc(allTransitions.^(1/10));
       xlabel('To Class'); ylabel('From Class'); 
       title(sprintf('Squeezed Transition Matrix with %d files', i));
       drawnow;
   end
end

%%

transitionProbabilities = allTransitions ./ ...
    repmat(sum(allTransitions,2),1, size(allTransitions,1));
classSums = sum(allTransitions,2);
% Regularize the class probabilities estimates....
classSums = classSums + mean(classSums(2:end))*pitchClassRegularizerRatio;
classProbabilities = classSums / sum(classSums);

clf
imagesc(transitionProbabilities.^(1/10));
xlabel('To State'); ylabel('From State');
title('All Pitch Class Transition Probabilities (Squeezed)');


