function [pitchTrackingErrors, voicedErrors, unvoicedErrors, bestOffsetList, ...
    lambdaList, priorRatios] = MeasurePitchPerformance(fileList, pitchModel)
% MEASUREPITCHPERFORMANCE Given a pitch model, calculate overall performance
%
% function [pitchTrackingError, voicedError, unvoicedError] = ...
%   MeasurePitchPerformance(fileList, pitchModel)
global gForceCorrelogramCalculation gLambdaList

if 1
    % This code makes sure that the transition matrix is full, and includes
    % all the states.  Original code didn't include all the high-order
    % states if they weren't seen in the training data. This makes all the
    % later process consistent.
    maxStates = max(QuantizePitches([10:10:1000]));
    at = pitchModel.allTransitions;
    at2 = zeros(maxStates, maxStates);
    at2(1:size(at,1), 1:size(at,2)) = at;
    pitchModel.allTransitions = at2;
end

priorRatios = [.1 .3 1 3 10 30 100];  % Regularize the prior class probabilities
if exist('gLambdaList','var') && ~isempty(gLambdaList)
    lambdaList = gLambdaList;
else
    lambdaList = [0 1e-3*(2).^(0:14)];  % Relative importance of observations vs. transitions
end

for i=1:length(fileList)            % Could be parfor....
    fprintf('Precomputing features for file %d/%d.\n', i, length(fileList));
    fileList(i) = ComputeAllFeatures(fileList(i), pitchModel);
end
originalForceCalculationValue = gForceCorrelogramCalculation;
gForceCorrelogramCalculation = 0;
fprintf('All done doing the ComputeAllFeatures.\n');
offsetList = -1:1;

pitchTrackingErrors = zeros(length(fileList), length(lambdaList), length(priorRatios));
unvoicedErrors = pitchTrackingErrors;
voicedErrors = pitchTrackingErrors;
bestOffsetList = [];

for pi=1:length(priorRatios)
    pitchClassRegularizerRatio = priorRatios(pi);
    
    % Regularize the class probabilities estimates....
    classSums = sum(pitchModel.allTransitions,2);
    classSums = classSums + mean(classSums(2:end))*pitchClassRegularizerRatio;
    classProbabilities = classSums / sum(classSums);
    pitchModel.classProbabilities = classProbabilities;
    
    bestOffsetList = zeros(length(fileList), length(lambdaList));
    pitchModel.lambda = lambdaList;
    pitchTrackingErrors(:,:,pi) = 0;
    for fi = 1:length(fileList)   
        % Now run the experiment
        groundTruthPitch = fileList(fi).pitchData;
        [viterbiPitch, ~ , pObservations, ~] = ...
            CalculatePitch(fileList(fi), pitchModel, groundTruthPitch);

        pteSlice = zeros(1,length(lambdaList));
        vSlice = pteSlice;          % Needed so parfor can understand;
        uvSlice = pteSlice;
        bSlice = pteSlice;
        for li=1:length(lambdaList)
            [pitchTrackingError, voicedError, unvoicedError, bestOffset] = ...
                PitchTrackingError(groundTruthPitch, viterbiPitch(li,:), ...
                    offsetList, pObservations);
            pteSlice(li) = pitchTrackingError;
            vSlice(li) = voicedError;
            uvSlice(li) = unvoicedError;
            % pitchTrackingErrors(fi, li, pi) = pitchTrackingError;
            % voicedErrors(fi,li,pi) = voicedError;
            % unvoicedErrors(fi,li,pi) = unvoicedError;
            
            % fprintf('Best offset for %s with lambda=%g is %d.\n', ...
            %     keeleFileList(fi).waveFile, lambdaList(li), bestOffset);
            bSlice(li) = bestOffset;
        end
        pitchTrackingErrors(fi,:,pi) = pteSlice;
        voicedErrors(fi,:,pi) = vSlice;
        unvoicedErrors(fi,:,pi) = uvSlice;
        bestOffsetList(fi,:) = bSlice;
    end
end
gForceCorrelogramCalculation = originalForceCalculationValue;

%%

clf
semilogx(lambdaList, squeeze(mean(pitchTrackingErrors,1)))
xlabel('Lambda (Transition Probability Weight)');
ylabel('Pitch Tracking Errors');
title('Pitch Performance vs. Prior Regularization');
legend('.1', '.3', '1', '3', '10', '30', '100', 'Location', 'NorthEast');


