function transformedFrames = PCATransform(data, pcaModel)
% PCATRANSFORM Reduce dimensionality of correlogram data with PCA
%
% function transformedFrames = PCATransform(data, pcaModel)
% Given some correlogram data, compute the transformed data after
% doing the PCA transform (subtract mean, correlate with eigenvectors).
%
% This code code can transform the data using the entire (correlogram)
% image as a vector (the image approach), or a channel-by-channel approach.
% The latter approach, channel-by-channel, is more effective on
% correlograms since channels are relatively independent.

if strcmp(pcaModel.pcaType, 'image')
    nPCA = pcaModel.nPCA;
    centeredFrames = data - repmat(pcaModel.pcaMeanValues, 1, size(data,2));
    transformedFrames = pcaModel.pcaVectors(:,1:nPCA)' * centeredFrames;
else
    nPCA = pcaModel.nPCA;
    nFrames = size(data, 2);
    frameSize = size(data, 1);
    nChannels = size(pcaModel.pcaVectors,3);
    % correlogramWidth = size(pcaModel.pcaVectors, 1);
    transformedFrames = zeros(nPCA*nChannels, nFrames);
    for c=1:size(pcaModel.pcaMeanValues, 2)
        centeredData = data(c:nChannels:frameSize,:) - ...
            repmat(pcaModel.pcaMeanValues(:,c), 1, nFrames);

        transformedData = squeeze(pcaModel.pcaVectors(:,1:nPCA, c))' * centeredData;
        transformedFrames(c:nChannels:nPCA*nChannels, :) = transformedData;
    end
end
