function [correlogramData, frameRate, numChannels] = ...
    PitchCorrelogram(inputSound, fs, frameRate, frameWidth)
% PITCHCORRELOGRAM Compute the correlogram needed for a pitch model
%
% function [correlogramData, frameRate, numChannels] = ...
%    PitchCorrelogram(inputSound, fs, frameRate, frameWidth)
% Convert sound into correlogram data (for a pitch experiment).
%% 
if nargin < 2
    fs = 16000;
end
if nargin < 3
    frameRate = 100;
end
if nargin < 4
    frameWidth = 256;
end

% Compute the channel-by-channel autocorrelation of the inputSound
lowPitch = 60;

global gShowCorrelogramFrames
gShowCorrelogramFrames = 0;

global gChannelCount
if exist('gChannelCount','var') && ~isempty(gChannelCount) && isfinite(gChannelCount)
    channelCount = gChannelCount;
else
    channelCount = 48;
end

if 0
    EarQ = 2*4;
    StepFactor = 0.5;

    coch=LyonPassiveEar(inputSound,3*fs,1,EarQ,StepFactor);
    [filters, freqs] = DesignLyonFilters(3*fs, EarQ, StepFactor);
    fprintf('Lowest frequency in auditory filterbank is %g\n', min(freqs));
else
    %%
    % Woops, MakeERBFilters always makes filters with the same bandwidth,
    % even as the number of channels change.  The new code figures out the
    % desired CFs, by spacing them equally in the log scale, and adjusts
    % the bandwidths. When there are fewer channels, we want the bandwidth
    % to go up.
    if 0
        lowFreq = 100;    logLow = log(lowFreq);
        highFreq = fs/2;  logHigh = log(highFreq);
        logCFs = logLow + (0.5:channelCount)*(logHigh-logLow)/channelCount;
        cfs = exp(logCFs);
        if channelCount == 1                % Bad hack since MakeERBFilters uses length of cfs
            cfs = [cfs cfs];
        end
    else
        cfs = channelCount;
    end
    fcoefs = MakeERBFiltersWide(fs, cfs, lowPitch/2, 48/channelCount);
    if channelCount == 1                    % Bad hack....
        fcoefs = fcoefs(1,:);
    end
    if 1                        % A little spectrum analysis for testing.
        impulse = [1 zeros(1,511)];
        resp = ERBFilterBank(impulse, fcoefs)';
        fresp = 20*log10(abs(fft(resp)));
        f = 0:fs/length(impulse):fs/2;
        semilogx(f, fresp(1:length(impulse)/2+1,:));
        xlabel('Frequency');
        ylabel('Filter Response (dB)');
        title('Gammatone Filterbank Frequency Response');
    end
    %%
    coch = max(0.0, ERBFilterBank(inputSound, fcoefs));
end

numChannels = size(coch, 1);

correlogramData = CorrelogramArray(coch,fs,frameRate,frameWidth);
