function q = QuantizePitches(p)
% QUANTIZEPITCHES Quantize all pitches into (70) discrete classes
%
% function q = QuantizePitches(p)
% Put all pitches into Dan Ellis' pitch classes.
% class 1 is unvoiced or no pitch (less than 20Hz in this code)
% class 2 is pitch less than lowPitch (60Hz)
% class 3 is pitch greater than highPitch (404Hz).
% The inverse of this function is DequantizePitch()
%
% See also:
%   DequantizePitches

q = 0*p;
noPitch =20;
lowPitch = 60;
highPitch = 404;
pitchClassesPerOctave = 24;

q(p < noPitch) = 1;
q((p >= noPitch) & (p < lowPitch)) = 2;
q(p > highPitch) = 3;
q(isnan(p)) = nan;
i = (q == 0);
q(i) = 4 + round(log2(p(i)/lowPitch) * pitchClassesPerOctave);
